/*---------------------------------------------------------------------------*\
 =========                   |
 \\      /   F ield          | OpenFOAM: The Open Source CFD Toolbox
  \\    /    O peration      |
   \\  /     A nd            | Copyright (C) 1991-2005 OpenCFD Ltd.
    \\/      M anipulation   |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

Application
    calculateTorque

Description
    calculates torque on boundary patches specified in dictionary
    file calculateTorqueDict for incompressible flow

\*---------------------------------------------------------------------------*/

/**
 * Basic OpenFOAM Finite Volume CFD Library
 */
#include "fvCFD.H"
#include "incompressible/singlePhaseTransportModel/singlePhaseTransportModel.H"
#include "RASModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
// Main program:


int main(int argc, char** argv)
{

#include "addTimeOptions.H"
#include "setRootCase.H"
#include "createTime.H"

// Get times list
instantList Times = runTime.times();

// set startTime and endTime depending on -time and -latestTime options
#include "checkTimeOptions.H"

runTime.setTime(Times[startTime], startTime);

#include "createMesh.H"
#include "GeometricFields.H"


//loop over time directories
for (label i=startTime; i<endTime; i++)
{
	runTime.setTime(Times[i], i);
	Info<< "Time = " << runTime.timeName() << "\n" << endl;

    /**
     * IODictionary: calculateTorqueDict
     * 
     */
	IOdictionary calculateTorqueDict
	(
		IOobject
		(
		"calculateTorqueDict",
		runTime.system(),
		mesh,
		IOobject::MUST_READ,
		IOobject::NO_WRITE
		)
	);


	/**
	 * Word list for the names of the patches to consider
	 */
    const wordList torquePatches(calculateTorqueDict.lookup("torquePatches"));


    /**
     * Calculating Angular velocity
     */
     #include "mathematicalConstants.H"
	scalar rpm = 0;
	scalar omega = 0;
	rpm = readScalar(calculateTorqueDict.lookup("rpm"));
	Info << "   Angular velocity:      rpm = "<< rpm << " [rev min^-1] "<< endl;
	omega = rpm*2.0*3.14159265/60.0;
	Info << "                        omega = "<< omega << " [s^-1] "<< endl;

     #include "createFields.H"


    /**
     *Calculating Torque
     */

	Info << " " << endl;
	Info << "   * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *" << endl;
	Info << "   *                 CALCULATION OF TORQUE                   *" << endl;
	Info << "   * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *" << endl;
	Info << " " << endl;


	//Calculation of wall shear stress tau

	volSymmTensorField Reff(RASModel->devReff());

        volVectorField wallShearStress
        (
            IOobject
            (
                "wallShearStress",
                runTime.timeName(),
                mesh,
                IOobject::NO_READ,
                IOobject::AUTO_WRITE
            ),
            mesh,
            dimensionedVector
            (
                "wallShearStress",
                Reff.dimensions(),
                vector::zero
            )
        );

	Info << "   Calculating wall shear stress \n" << endl;
        forAll(wallShearStress.boundaryField(), patchi)
        {
            wallShearStress.boundaryField()[patchi] =
            (
                -mesh.Sf().boundaryField()[patchi]
                /mesh.magSf().boundaryField()[patchi]
            ) & Reff.boundaryField()[patchi];
        }
        wallShearStress.write();      // [m^2 s^-2]


	Info << "   Torque is calculated on following patches:" << endl;
	Info << " " << endl;

vector totalTorque (0,0,0);

forAll(torquePatches,tpI)
{
	label patchIndexT=mesh.boundaryMesh().findPatchID(torquePatches[tpI]);

    //Check whether patch exists
    if (patchIndexT > -1)
    {

	// Check whether patch has type wall
 	if (mesh.boundary()[patchIndexT].type()=="wall")
	{

	//General patch information
		Info << "   ---------------------------------------------------------------"<<endl;
		Info << "   patch name: "<< torquePatches[tpI] <<endl;
		Info << "   patch ID:   "<< mesh.boundaryMesh().findPatchID(torquePatches[tpI]) <<endl;
		Info << "   patch type: "<< mesh.boundary()[patchIndexT].type()<<endl;
		Info << "   faces:      "<< mesh.boundary()[patchIndexT].faceCells().size()<<endl;


	//Calculating patch area
		scalar A=gSum(mesh.boundary()[patchIndexT].magSf());
		Info << "      area A       = "<< A << " [m^2]" << endl;


	//Calculating torque = r x F, F = p*A + tau*A
	vector torque (0,0,0);
	const fvPatch& cPatch = mesh.boundary()[patchIndexT];

	torque = gSum((cPatch.Cf())^(((p.boundaryField()[patchIndexT])*cPatch.Sf()) - (wallShearStress.boundaryField()[patchIndexT])*cPatch.magSf()));
        totalTorque=totalTorque+torque;


	Info << "      torque(" << torquePatches[tpI] << "): " << torque << " [Nm*m^3/kg]" << endl;
	Info << "      power:                      " << torque.z()*omega << " [W*m^3/kg]" << endl;
	}
	else
	{
		Info << "   ---------------------------------------------------------------"<<endl;
	        Info << "   Patch " << torquePatches[tpI] << " is not a wall!" << endl;
	}
    }
    else
    {
		Info << "   ---------------------------------------------------------------"<<endl;
	        Info << "   Patch " << torquePatches[tpI] << " not found!" << endl;

    }

}
		Info << "   ---------------------------------------------------------------"<<endl;

    
		Info << "   ===============================================================" << endl;
		Info << "      total torque: " << totalTorque << " [Nm*m^3/kg]" << endl;
		Info << "   ===============================================================" << endl;
		Info << "      total power:                   " <<  totalTorque.z()*omega << " [W*m^3/kg]" << endl;
		Info << "   ===============================================================\n" << endl;


}


return 0;
}
